body {
  margin: 2em 1em 2em 70px;
  font-family: sans-serif;
  color: black;
  background: white;
  background-position: top left;
  background-attachment: fixed;
  background-repeat: no-repeat;
}

th, td { /* ns 4 */
  font-family: sans-serif;
}

h1, h2, h3, h4, h5, h6 { text-align: left }
h1, h2, h3 { color: #005A9C }
h1 { font: 150% sans-serif }
h2 { font: 130% sans-serif }
h3 { font: 110% sans-serif }
h4 { font: bold 100% sans-serif }
h5 { font: italic 100% sans-serif }
